// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef AUDIO_INCLUDE_ES_AUDIO_H_
#define AUDIO_INCLUDE_ES_AUDIO_H_

#include "es_type.h"
#include "es_common.h"
#include "es_comm_aio.h"
#include "es_comm_aenc.h"
#include "es_comm_adec.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* __cplusplus */

/**
 * @brief Sets the public attributes for an audio input device.
 *
 * This function sets the public attributes (such as sample rate, bit depth, etc.)
 * for the specified audio input device.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[in] Attr Pointer to a structure containing the desired audio input attributes.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_SetPubAttr(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, const AIO_ATTR_S *Attr);

/**
 * @brief Gets the public attributes of an audio input device.
 *
 * This function retrieves the public attributes (such as sample rate, bit depth, etc.)
 * of the specified audio input device.
 *
 * @param[in] AiDevId The ID of the audio input device.
 * @param[out] Attr Pointer to a structure where the audio input attributes will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_GetPubAttr(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, AIO_ATTR_S *Attr);

/**
 * @brief Enables an audio input device.
 *
 * This function enables the specified audio input device for capturing audio data.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_Enable(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId);

/**
 * @brief Disables an audio input device.
 *
 * This function disables the specified audio input device, stopping the capture of audio data.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_Disable(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId);

/**
 * @brief Enables an audio input channel of a specific device.
 *
 * This function enables the specified audio input channel for capturing audio data from a device.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_EnableChn(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId);

/**
 * @brief Disables an audio input channel of a specific device.
 *
 * This function disables the specified audio input channel, stopping the capture of audio data from a device.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_DisableChn(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId);

/**
 * @brief Gets an audio frame from a specific audio input channel of a device.
 *
 * This function retrieves an audio frame from the specified audio input channel of the device.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[out] Frame Pointer to a structure where the audio frame will be stored.
 * @param[out] AecFrame Pointer to a structure where the AEC audio frame will be stored (if available).
 * @param[in] MilliSec The timeout value in milliseconds for waiting for a frame.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_GetFrame(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, AUDIO_FRAME_S *Frame, AUDIO_AEC_FRAME_S *AecFrame,
                      ES_S32 MilliSec);

/**
 * @brief Releases an audio frame obtained from a specific audio input channel of a device.
 *
 * This function releases the resources associated with the audio frame obtained from the specified audio input channel.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[in] Frame Pointer to the audio frame that needs to be released.
 * @param[in] AecFrame Pointer to the AEC audio frame that needs to be released (if available).
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_ReleaseFrame(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, const AUDIO_FRAME_S *Frame,
                          const AUDIO_AEC_FRAME_S *AecFrame);

/**
 * @brief Sets the parameters of a specific audio input channel of a device.
 *
 * This function sets the parameters (such as volume, sample rate, etc.) of the specified audio input channel.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[in] ChnParam Pointer to a structure containing the audio input channel parameters.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_SetChnParam(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, const AI_CHN_PARAM_S *ChnParam);

/**
 * @brief Gets the parameters of a specific audio input channel of a device.
 *
 * This function retrieves the parameters (such as volume, sample rate, etc.) of the specified audio input channel.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[out] ChnParam Pointer to a structure where the audio input channel parameters will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_GetChnParam(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, AI_CHN_PARAM_S *ChnParam);

/**
 * @brief Sets the VQE (Voice Quality Enhancement) attributes for a specific audio input channel of a device.
 *
 * This function configures the VQE parameters for the specified audio input channel.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[in] VqeConfig Pointer to a structure containing the VQE configuration.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_SetVqeAttr(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, const AI_VQE_CONFIG_S *VqeConfig);

/**
 * @brief Gets the VQE (Voice Quality Enhancement) attributes for a specific audio input channel of a device.
 *
 * This function retrieves the VQE parameters configured for the specified audio input channel.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[out] VqeConfig Pointer to a structure where the VQE configuration will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_GetVqeAttr(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, AI_VQE_CONFIG_S *VqeConfig);

/**
 * @brief Enables VQE (Voice Quality Enhancement) for a specific audio input channel of a device.
 *
 * This function enables VQE processing for the specified audio input channel.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_EnableVqe(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId);

/**
 * @brief Disables VQE (Voice Quality Enhancement) for a specific audio input channel of a device.
 *
 * This function disables VQE processing for the specified audio input channel.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_DisableVqe(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId);

/**
 * @brief Enables resampling for a specific audio input channel of a device.
 *
 * This function enables resampling for the specified audio input channel, with the specified output sample rate.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[in] OutSampleRate The desired output sample rate after resampling.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_EnableReSmp(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, AUDIO_SAMPLE_RATE_E OutSampleRate);

/**
 * @brief Disables resampling for a specific audio input channel of a device.
 *
 * This function disables resampling for the specified audio input channel.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_DisableReSmp(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId);

/**
 * @brief Sets the track mode for audio input.
 *
 * This function sets the track mode for audio input. The track mode determines the type of audio data to be captured.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[in] TrackMode The track mode to be set.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_SetTrackMode(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, AUDIO_TRACK_MODE_E TrackMode);

/**
 * @brief Gets the track mode for audio input.
 *
 * This function retrieves the current track mode set for audio input.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[out] TrackMode Pointer to a variable where the current track mode will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_GetTrackMode(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, AUDIO_TRACK_MODE_E *TrackMode);

/**
 * @brief Saves audio data to a file for a specific audio input channel of a device.
 *
 * This function saves audio data from the specified audio input channel to a file, according to the provided file
 * information.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[in] SaveFileInfo Pointer to a structure containing information about saving the audio data to a file.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_SaveFile(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, const AUDIO_SAVE_FILE_INFO_S *SaveFileInfo);

/**
 * @brief Queries the status of the audio file operation for a specific audio input channel of a device.
 *
 * This function retrieves the status of the audio file operation for the specified audio input channel.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[out] FileStatus Pointer to a structure where the file operation status will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_QueryFileStatus(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, AUDIO_FILE_STATUS_S *FileStatus);

/**
 * @brief Clears the public attributes of an audio input device.
 *
 * This function clears the public attributes (such as sample rate, bit depth, etc.) of the specified audio input
 * device.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_ClrPubAttr(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId);

/**
 * @brief Gets the file descriptor associated with an audio input channel of a device.
 *
 * This function retrieves the file descriptor associated with the specified audio input channel of the device.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @return The file descriptor on success, a negative error code otherwise.
 */
ES_S32 ES_AI_GetFd(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId);

/**
 * @brief Enables AEC (Acoustic Echo Cancellation) reference frame for an audio input channel.
 *
 * This function enables the use of a specific audio channel from an audio output device as a reference frame for AEC
 * processing in the audio input channel.
 *
 * @param[in] AiCardId The ID of the audio input card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[in] AoCardId The ID of the audio output card providing the reference frame.
 * @param[in] AoDevId The ID of the audio output device providing the reference frame.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_EnableAecRefFrame(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Disables AEC (Acoustic Echo Cancellation) reference frame for an audio input channel.
 *
 * This function disables the use of a specific audio channel from an audio output device as a reference frame for AEC
 * processing in the audio input channel.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_DisableAecRefFrame(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId);

/**
 * @brief Sets the volume level for an audio input device.
 *
 * This function sets the volume level (in decibels) for the specified audio input device.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[in] VolumeDb The desired volume level in decibels.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_SetVolume(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, ES_S32 VolumeDb);

/**
 * @brief Gets the current volume level of an audio input device.
 *
 * This function retrieves the current volume level (in decibels) of the specified audio input device.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AiDevId The ID of the audio input device.
 * @param[out] VolumeDb Pointer to a variable where the current volume level will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AI_GetVolume(AUDIO_CARD AiCardId, AUDIO_DEV AiDevId, ES_S32 *VolumeDb);

/**
 * @brief Sets the public attributes of an audio output device.
 *
 * This function sets the public attributes (such as sample rate, bit depth, etc.) of the specified audio output device.
 *
 * @param[in] AiCardId The ID of the audio card.
 * @param[in] AoDevId The ID of the audio output device.
 * @param[in] Attr Pointer to a structure containing the audio output attributes.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_SetPubAttr(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, const AIO_ATTR_S *Attr);

/**
 * @brief Gets the public attributes of an audio output device.
 *
 * This function retrieves the public attributes (such as sample rate, bit depth, etc.) of the specified audio output
 * device.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @param[out] Attr Pointer to a structure where the audio output attributes will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_GetPubAttr(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, AIO_ATTR_S *Attr);

/**
 * @brief Enables an audio output device.
 *
 * This function enables the specified audio output device for audio playback.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_Enable(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Disables an audio output device.
 *
 * This function disables the specified audio output device, stopping audio playback.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_Disable(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Enables an audio output channel of a specific device.
 *
 * This function enables the specified audio output channel for audio playback.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_EnableChn(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Disables an audio output channel of a specific device.
 *
 * This function disables the specified audio output channel, stopping audio playback on that channel.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_DisableChn(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Sends an audio frame to an audio output channel.
 *
 * This function sends an audio frame to the specified audio output channel for playback.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @param[in] Frame Pointer to the audio frame to be sent.
 * @param[in] MilliSec The timeout value in milliseconds for waiting to send the frame.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_SendFrame(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, const AUDIO_FRAME_S *Frame, ES_S32 MilliSec);

/**
 * @brief Enables resampling for an audio output channel of a device.
 *
 * This function enables resampling for the specified audio output channel, with the specified input sample rate.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @param[in] InSampleRate The input sample rate before resampling.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_EnableReSmp(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, AUDIO_SAMPLE_RATE_E InSampleRate);

/**
 * @brief Disables resampling for an audio output channel of a device.
 *
 * This function disables resampling for the specified audio output channel.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_DisableReSmp(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Clears the buffer of an audio output channel.
 *
 * This function clears the buffer of the specified audio output channel.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_ClearChnBuf(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Queries the status of an audio output channel.
 *
 * This function retrieves the status of the specified audio output channel, such as whether it is running or paused.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @param[out] Status Pointer to a structure where the channel status will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_QueryChnStat(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, AO_CHN_STATE_S *Status);

/**
 * @brief Pauses an audio output channel.
 *
 * This function pauses audio playback on the specified audio output channel.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_PauseChn(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Resumes audio playback on a paused audio output channel.
 *
 * This function resumes audio playback on the specified audio output channel that was previously paused.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_ResumeChn(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Sets the volume level for an audio output device.
 *
 * This function sets the volume level (in decibels) for the specified audio output device.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @param[in] VolumeDb The desired volume level in decibels.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_SetVolume(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, ES_S32 VolumeDb);

/**
 * @brief Gets the current volume level of an audio output device.
 *
 * This function retrieves the current volume level (in decibels) of the specified audio output device.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @param[out] VolumeDb Pointer to a variable where the current volume level will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_GetVolume(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, ES_S32 *VolumeDb);

/**
 * @brief Sets mute status for an audio output device.
 *
 * This function enables or disables mute for the specified audio output device and optionally applies a fade effect.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @param[in] Enable ES_TRUE to enable mute, ES_FALSE to disable mute.
 * @param[in] Fade Pointer to a structure containing fade parameters (optional, can be NULL).
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_SetMute(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, ES_BOOL Enable, const AUDIO_FADE_S *Fade);

/**
 * @brief Gets mute status for an audio output device.
 *
 * This function retrieves the current mute status of the specified audio output device, and optionally the fade effect
 * parameters.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @param[out] Enable Pointer to a variable where the mute status will be stored.
 * @param[out] Fade Pointer to a structure where the fade effect parameters will be stored (optional, can be NULL).
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_GetMute(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, ES_BOOL *Enable, AUDIO_FADE_S *Fade);

/**
 * @brief Sets the track mode for audio output.
 *
 * This function sets the track mode for audio output. The track mode determines the type of audio data to be played.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @param[in] TrackMode The track mode to be set.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_SetTrackMode(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, AUDIO_TRACK_MODE_E TrackMode);

/**
 * @brief Gets the track mode for audio output.
 *
 * This function retrieves the current track mode set for audio output.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @param[out] TrackMode Pointer to a variable where the current track mode will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_GetTrackMode(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, AUDIO_TRACK_MODE_E *TrackMode);

/**
 * @brief Gets the file descriptor associated with an audio output channel of a device.
 *
 * This function retrieves the file descriptor associated with the specified audio output channel of the device.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @return The file descriptor on success, a negative error code otherwise.
 */
ES_S32 ES_AO_GetFd(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Saves audio data from an audio output channel to a file for a specific audio input channel of a device.
 *
 * This function saves audio data from the specified audio output channel to a file, according to the provided file
 * information.
 *
 * @param[in] AiDevId The ID of the audio input device.
 * @param[in] SaveFileInfo Pointer to a structure containing information about saving the audio data to a file.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_SaveFile(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, const AUDIO_SAVE_FILE_INFO_S *SaveFileInfo);

/**
 * @brief Queries the status of the audio file operation for a specific audio output channel of a device.
 *
 * This function retrieves the status of the audio file operation for the specified audio output channel.
 *
 * @param[in] AiDevId The ID of the audio input device.
 * @param[out] FileStatus Pointer to a structure where the file operation status will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_QueryFileStatus(AUDIO_CARD AoCardId, AUDIO_DEV AiDevId, AUDIO_FILE_STATUS_S *FileStatus);

/**
 * @brief Clears the public attributes of an audio output device.
 *
 * This function clears the public attributes (such as sample rate, bit depth, etc.) of the specified audio output
 * device.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_ClrPubAttr(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Sets the VQE (Voice Quality Enhancement) attributes for a specific audio output channel of a device.
 *
 * This function configures the VQE parameters for the specified audio output channel.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @param[in] VqeConfig Pointer to a structure containing the VQE configuration.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_SetVqeAttr(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, const AO_VQE_CONFIG_S *VqeConfig);

/**
 * @brief Gets the VQE (Voice Quality Enhancement) attributes for a specific audio output channel of a device.
 *
 * This function retrieves the VQE parameters configured for the specified audio output channel.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @param[out] VqeConfig Pointer to a structure where the VQE configuration will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_GetVqeAttr(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, AO_VQE_CONFIG_S *VqeConfig);

/**
 * @brief Enables VQE (Voice Quality Enhancement) for a specific audio output channel of a device.
 *
 * This function enables VQE processing for the specified audio output channel.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_EnableVqe(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Disables VQE (Voice Quality Enhancement) for a specific audio output channel of a device.
 *
 * This function disables VQE processing for the specified audio output channel.
 *
 * @param[in] AoDevId The ID of the audio output device.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AO_DisableVqe(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId);

/**
 * @brief Creates an audio encoder channel.
 *
 * This function creates an audio encoder channel with the specified attributes.
 *
 * @param[in] AeChn The ID of the audio encoder channel to be created.
 * @param[in] Attr Pointer to a structure containing the attributes for the audio encoder channel.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_CreateChn(AENC_CHN AeChn, const AENC_CHN_ATTR_S *Attr);

/**
 * @brief Destroys an audio encoder channel.
 *
 * This function destroys the specified audio encoder channel.
 *
 * @param[in] AeChn The ID of the audio encoder channel to be destroyed.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_DestroyChn(AENC_CHN AeChn);

/**
 * @brief Sends an audio frame to an audio encoder channel.
 *
 * This function sends an audio frame to the specified audio encoder channel for encoding.
 *
 * @param[in] AeChn The ID of the audio encoder channel.
 * @param[in] Frame Pointer to the audio frame to be encoded.
 * @param[in] AecFrame Pointer to the audio AEC (Acoustic Echo Cancellation) frame (optional, can be NULL).
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_SendFrame(AENC_CHN AeChn, const AUDIO_FRAME_S *Frame, const AUDIO_AEC_FRAME_S *AecFrame);

/**
 * @brief Gets an audio stream from an audio encoder channel.
 *
 * This function retrieves an encoded audio stream from the specified audio encoder channel.
 *
 * @param[in] AeChn The ID of the audio encoder channel.
 * @param[out] Stream Pointer to a structure where the audio stream will be stored.
 * @param[in] MilliSec The timeout value in milliseconds for waiting to get the stream.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_GetStream(AENC_CHN AeChn, AUDIO_STREAM_S *Stream, ES_S32 MilliSec);

/**
 * @brief Releases an audio stream obtained from an audio encoder channel.
 *
 * This function releases an audio stream obtained from the specified audio encoder channel.
 *
 * @param[in] AeChn The ID of the audio encoder channel.
 * @param[in] Stream Pointer to the audio stream to be released.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_ReleaseStream(AENC_CHN AeChn, const AUDIO_STREAM_S *Stream);

/**
 * @brief Gets the file descriptor associated with an audio encoder channel.
 *
 * This function retrieves the file descriptor associated with the specified audio encoder channel.
 *
 * @param[in] AeChn The ID of the audio encoder channel.
 * @return The file descriptor on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_GetFd(AENC_CHN AeChn);

/**
 * @brief Registers a custom audio encoder.
 *
 * This function registers a custom audio encoder with the system.
 *
 * @param[out] Handle Pointer to a variable where the handle for the registered encoder will be stored.
 * @param[in] Encoder Pointer to a structure containing the custom audio encoder information.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_RegisterEncoder(ES_S32 *Handle, const AENC_ENCODER_S *Encoder);

/**
 * @brief Unregisters a custom audio encoder.
 *
 * This function unregisters a custom audio encoder from the system.
 *
 * @param[in] Handle The handle of the registered encoder to be unregistered.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_UnRegisterEncoder(ES_S32 Handle);

/**
 * @brief Saves audio data from an audio encoder channel to a file.
 *
 * This function saves audio data from the specified audio encoder channel to a file, according to the provided file
 * information.
 *
 * @param[in] AeChn The ID of the audio encoder channel.
 * @param[in] SaveFileInfo Pointer to a structure containing information about saving the audio data to a file.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_SaveFile(AENC_CHN AeChn, const AUDIO_SAVE_FILE_INFO_S *SaveFileInfo);

/**
 * @brief Queries the status of the audio file operation for an audio encoder channel.
 *
 * This function retrieves the status of the audio file operation for the specified audio encoder channel.
 *
 * @param[in] AeChn The ID of the audio encoder channel.
 * @param[out] FileStatus Pointer to a structure where the file operation status will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_QueryFileStatus(AENC_CHN AeChn, AUDIO_FILE_STATUS_S *FileStatus);

/**
 * @brief Gets information about the stream buffer for an audio encoder channel.
 *
 * This function retrieves the physical address and size of the stream buffer associated with the specified audio
 * encoder channel.
 *
 * @param[in] AeChn The ID of the audio encoder channel.
 * @param[out] PhysAddr Pointer to a variable where the physical address of the stream buffer will be stored.
 * @param[out] Size Pointer to a variable where the size of the stream buffer will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_GetStreamBufInfo(AENC_CHN AeChn, ES_U64 *PhysAddr, ES_U32 *Size);

/**
 * @brief Sets mute status for an audio encoder channel.
 *
 * This function enables or disables mute for the specified audio encoder channel.
 *
 * @param[in] AeChn The ID of the audio encoder channel.
 * @param[in] Enable ES_TRUE to enable mute, ES_FALSE to disable mute.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_SetMute(AENC_CHN AeChn, ES_BOOL Enable);

/**
 * @brief Gets mute status for an audio encoder channel.
 *
 * This function retrieves the current mute status of the specified audio encoder channel.
 *
 * @param[in] AeChn The ID of the audio encoder channel.
 * @param[out] Enable Pointer to a variable where the mute status will be stored.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_AENC_GetMute(AENC_CHN AeChn, ES_BOOL *Enable);

/**
 * @brief Creates an audio decoder channel.
 *
 * This function creates an audio decoder channel with the specified attributes.
 *
 * @param[in] AdChn The ID of the audio decoder channel to be created.
 * @param[in] Attr Pointer to a structure containing the attributes for the audio decoder channel.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_ADEC_CreateChn(ADEC_CHN AdChn, const ADEC_CHN_ATTR_S *Attr);

/**
 * @brief Destroys an audio decoder channel.
 *
 * This function destroys the specified audio decoder channel.
 *
 * @param[in] AdChn The ID of the audio decoder channel to be destroyed.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_ADEC_DestroyChn(ADEC_CHN AdChn);

/**
 * @brief Sends an audio stream to an audio decoder channel.
 *
 * This function sends an audio stream to the specified audio decoder channel for decoding.
 *
 * @param[in] AdChn The ID of the audio decoder channel.
 * @param[in] Stream Pointer to the audio stream to be decoded.
 * @param[in] Block ES_TRUE to block until the operation is complete, ES_FALSE for non-blocking.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_ADEC_SendStream(ADEC_CHN AdChn, const AUDIO_STREAM_S *Stream, ES_BOOL Block);

/**
 * @brief Clears the buffer of an audio decoder channel.
 *
 * This function clears the buffer of the specified audio decoder channel.
 *
 * @param[in] AdChn The ID of the audio decoder channel.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_ADEC_ClearChnBuf(ADEC_CHN AdChn);

/**
 * @brief Registers a custom audio decoder.
 *
 * This function registers a custom audio decoder with the system.
 *
 * @param[out] Handle Pointer to a variable where the handle for the registered decoder will be stored.
 * @param[in] Decoder Pointer to a structure containing the custom audio decoder information.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_ADEC_RegisterDecoder(ES_S32 *Handle, const ADEC_DECODER_S *Decoder);

/**
 * @brief Unregisters a custom audio decoder.
 *
 * This function unregisters a custom audio decoder from the system.
 *
 * @param[in] Handle The handle of the registered decoder to be unregistered.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_ADEC_UnRegisterDecoder(ES_S32 Handle);

/**
 * @brief Gets an audio frame from an audio decoder channel.
 *
 * This function retrieves an audio frame from the specified audio decoder channel.
 *
 * @param[in] AdChn The ID of the audio decoder channel.
 * @param[out] FrameInfo Pointer to a structure where the audio frame information will be stored.
 * @param[in] Block ES_TRUE to block until a frame is available, ES_FALSE for non-blocking.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_ADEC_GetFrame(ADEC_CHN AdChn, AUDIO_FRAME_INFO_S *FrameInfo, ES_BOOL Block);

/**
 * @brief Releases an audio frame obtained from an audio decoder channel.
 *
 * This function releases an audio frame obtained from the specified audio decoder channel.
 *
 * @param[in] AdChn The ID of the audio decoder channel.
 * @param[in] FrameInfo Pointer to the audio frame information to be released.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_ADEC_ReleaseFrame(ADEC_CHN AdChn, const AUDIO_FRAME_INFO_S *FrameInfo);

/**
 * @brief Sends an end-of-stream indication to an audio decoder channel.
 *
 * This function sends an end-of-stream indication to the specified audio decoder channel.
 *
 * @param[in] AdChn The ID of the audio decoder channel.
 * @param[in] Instant ES_TRUE for instant end-of-stream, ES_FALSE for normal end-of-stream.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_ADEC_SendEndOfStream(ADEC_CHN AdChn, ES_BOOL Instant);

/**
 * @brief Sets a value for the specified audio codec control on a specific audio device.
 *
 * This function sets the value for the specified audio codec control on the specified audio device.
 *
 * @param[in] DevId The ID of the audio device.
 * @param[in] Control The name of the audio codec control.
 * @param[in] Value The value to be set for the control.
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_ACODEC_SetValue(AUDIO_DEV DevId, const char *Control, const char *Value);

/**
 * @brief Gets the current value of the specified audio codec control on a specific audio device.
 *
 * This function retrieves the current value of the specified audio codec control on the specified audio device.
 *
 * @param[in] DevId The ID of the audio device.
 * @param[in] Control The name of the audio codec control.
 * @param[out] Value The buffer to store the retrieved value.
 * @param[in]  Size    The size of the buffer (in bytes).
 * @return ES_SUCCESS on success, a negative error code otherwise.
 */
ES_S32 ES_ACODEC_GetValue(AUDIO_DEV DevId, const char *Control, char *Value, ES_U32 Size);

/**
 * @brief Get Audio version information.
 *
 * @param[out] version Version number of the Audio.
 * @return Returns the execution status code: ES_SUCCESS for success, others for failure.
 */
ES_S32 ES_Audio_GetVersion(ES_U64 *version);

/**
 * @brief Get Audio version string.
 *
 * @param[out] version Buffer to store the version information
 * @param[in] maxSize Maximum size of the buffer
 * @return Returns the status code of the operation; 0 if successful, an error code otherwise
 */
ES_S32 ES_Audio_GetVersionString(ES_CHAR *version, ES_U32 maxSize);

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif  // AUDIO_INCLUDE_ES_AUDIO_H_
