// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef AUDIO_INCLUDE_ES_COMM_ADEC_H_
#define AUDIO_INCLUDE_ES_COMM_ADEC_H_

#include "es_type.h"
#include "es_common.h"
#include "es_comm_aio.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif

typedef struct {
    ES_BOOL IsULaw;
} ADEC_ATTR_G711_S;

typedef struct {
    ES_S32 BitRate;
} ADEC_ATTR_G722_S;

typedef struct {
    ES_S32 BitRate;
} ADEC_ATTR_G726_S;

typedef struct {
    ES_BOOL IsWb;
} ADEC_ATTR_AMR_S;

typedef enum {
    ADEC_AAC_TRANS_TYPE_ADTS = 0,
    ADEC_AAC_TRANS_TYPE_LOAS = 1,
    ADEC_AAC_TRANS_TYPE_LATM_MCP1 = 2,
    ADEC_AAC_TRANS_TYPE_BUTT
} ADEC_AAC_TRANS_TYPE_E;

typedef enum {
    ADEC_AAC_TYPE_AACMAIN = 0,
    ADEC_AAC_TYPE_AACLC = 1,
    ADEC_AAC_TYPE_AACHE = 2,
    ADEC_AAC_TYPE_AACLD = 3,
    ADEC_AAC_TYPE_BUTT,
} ADEC_AAC_PROFILE_E;

typedef struct {
    ADEC_AAC_PROFILE_E Profile;
    ADEC_AAC_TRANS_TYPE_E TransType;
    ES_S32 OutputFormat;
} ADEC_ATTR_AAC_S;

typedef struct {
    ES_U8 *Input;
    ES_S32 InputSize;
    ES_S32 FrameSize;
    ES_S32 FrameOffset;
    ES_U32 SampleRate;
    ES_S16 Channels;
    ES_S16 BitDepth;
} ADEC_FRAME_INFO;

typedef enum {
    /**
     * require input is valid dec pack(a complete frame encode result), e.g.the stream get from AENC is a
     * valid dec pack, the stream know actually pack len from file is also a dec pack. this mode is high-performative.
     */
    ADEC_MODE_PACK = 0,
    /**
     * input is stream, low-performative, if you couldn't find out whether
     * a stream is vaild dec pack,you could use this mode.
     */
    ADEC_MODE_STREAM,
    ADEC_MODE_BUTT
} ADEC_MODE_E;

typedef struct {
    PAYLOAD_TYPE_E Type;
    ES_U32 BufSize;   /* buf size[2~MAX_AUDIO_FRAME_NUM] */
    ADEC_MODE_E Mode; /* decode mode */
    ES_VOID ATTRIBUTE *Value;
} ADEC_CHN_ATTR_S;

typedef struct {
    PAYLOAD_TYPE_E Type;
    ES_CHAR Name[17];
    /*struct ppDecoder is packed by user, user malloc and free memory for this struct */
    ES_S32 (*FunOpenDecoder)(ES_VOID *DecoderAttr, ES_VOID **Decoder);
    ES_S32(*FunDecodeFrm)
    (ES_VOID *Decoder, const ES_U8 *InputBuffer, ES_S32 InputSize, ES_U8 *OutBuffer, ES_U32 *OutSize);
    ES_S32 (*FunGetFrmInfo)(ES_VOID *Decoder, ES_VOID *Info);
    ES_S32 (*FunCloseDecoder)(ES_VOID *Decoder);
    ES_S32 (*FunResetDecoder)(ES_VOID *Decoder);
} ADEC_DECODER_S;

typedef enum {
    ADEC_ERR_DECODER_ERR = 64,
    ADEC_ERR_BUF_LACK,
} ADEC_ERR_CODE_E;

/* output format 16bit */
#define ADEC_FMT_16BIT 1
/* output format 24bit */
#define ADEC_FMT_24BIT 2
/* output format 32bit */
#define ADEC_FMT_32BIT 3

#define ADEC_MAX_CHN_NUM 16
/* invlalid device ID */
#define ADEC_ERR_INVALID_DEVID ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_DEVID)
/* invlalid channel ID */
#define ADEC_ERR_INVALID_CHNID ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_CHNID)
/* at lease one parameter is illagal ,eg, an illegal enumeration value  */
#define ADEC_ERR_ILLEGAL_PARAM ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_ILLEGAL_PARAM)
/* channel exists */
#define ADEC_ERR_EXIST ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_EXIST)
/* channel unexists */
#define ADEC_ERR_UNEXIST ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_UNEXIST)
/* using a NULL point */
#define ADEC_ERR_NULL_PTR ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_NULL_PTR)
/* try to enable or initialize system,device or channel, before configing attribute */
#define ADEC_ERR_NOT_CONFIG ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_CONFIG)
/* operation is not supported by NOW */
#define ADEC_ERR_NOT_SUPPORT ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_SUPPORT)
/* operation is not permitted ,eg, try to change stati attribute */
#define ADEC_ERR_NOT_PERM ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_PERM)
/* failure caused by malloc memory */
#define ADEC_ERR_NOMEM ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_NOMEM)
/* failure caused by malloc buffer */
#define ADEC_ERR_NOBUF ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_NOBUF)
/* no data in buffer */
#define ADEC_ERR_BUF_EMPTY ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_EMPTY)
/* no buffer for new data */
#define ADEC_ERR_BUF_FULL ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_FULL)
/* system is not ready,had not initialed or loaded*/
#define ADEC_ERR_SYS_NOTREADY ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, EN_ERR_SYS_NOTREADY)
/* decoder internal err */
#define ADEC_ERR_DECODER_ERR ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, ADEC_ERR_DECODER_ERR)
/* input buffer not enough to decode one frame */
#define ADEC_ERR_BUF_LACK ES_DEF_ERR(ES_ID_ADEC, EN_ERR_LEVEL_ERROR, ADEC_ERR_BUF_LACK)

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif

#endif  // AUDIO_INCLUDE_ES_COMM_ADEC_H_
