// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef AUDIO_INCLUDE_ES_COMM_AENC_H_
#define AUDIO_INCLUDE_ES_COMM_AENC_H_

#include "es_type.h"
#include "es_common.h"
#include "es_comm_aio.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif

typedef struct {
    ES_S32 BitRate;
    ES_S8 Aot;
    ES_S8 Sbr;
    ES_S8 Channels;
    ES_S8 Vbr;
    ES_S32 SampleRate;
    ES_S32 Afterburner;
} AENC_ATTR_AAC_S;

typedef struct {
    ES_BOOL IsULaw;
} AENC_ATTR_G711_S;

typedef struct {
    ES_S32 BitRate;
} AENC_ATTR_G722_S;

typedef struct {
    ES_S32 BitRate;
} AENC_ATTR_G726_S;

typedef struct {
    /**
     * NB:4750/5150/5900/6700/7400/7950/10200/12200 bps
     * WB:6600/8850/12650/14250/15850/18250/19850/23050/23850 bps
     */
    ES_S32 BitRate;
    /**
     * 0:disable DTX; 1: enable DTX
     */
    ES_S32 Dtx;
    ES_S32 IsWb;
} AENC_ATTR_AMR_S;

typedef struct {
    PAYLOAD_TYPE_E Type;
    ES_U32 MaxFrmLen;
    ES_CHAR Name[17]; /* encoder type,be used to print proc information */
    ES_S32(*FunOpenEncoder)
    (ES_VOID *EncoderAttr, ES_VOID **Encoder); /* pEncoder is the handle to control the encoder */
    ES_S32 (*FunEncodeFrm)(ES_VOID *Encoder, const ES_U8 *Input, ES_U32 InputSize, ES_U8 *Output, ES_U32 *OutSize);
    ES_S32 (*FunCloseEncoder)(ES_VOID *Encoder);
} AENC_ENCODER_S;

typedef struct {
    PAYLOAD_TYPE_E Type; /*payload type ()*/
    ES_U32 PtNumPerFrm;
    ES_U32 BufSize;           /*buf size [2~MAX_AUDIO_FRAME_NUM]*/
    ES_VOID ATTRIBUTE *Value; /*point to attribute of definite audio encoder*/
} AENC_CHN_ATTR_S;

typedef enum {
    AENC_ERR_ENCODER_ERR = 64,
    AENC_ERR_VQE_ERR = 65,
} AENC_ERR_CODE_E;

#define AENC_MAX_CHN_NUM 8
/* invlalid device ID */
#define AENC_ERR_INVALID_DEVID ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_DEVID)
/* invlalid channel ID */
#define AENC_ERR_INVALID_CHNID ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_CHNID)
/* at lease one parameter is illagal ,eg, an illegal enumeration value  */
#define AENC_ERR_ILLEGAL_PARAM ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_ILLEGAL_PARAM)
/* channel exists */
#define AENC_ERR_EXIST ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_EXIST)
/* channel unexists */
#define AENC_ERR_UNEXIST ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_UNEXIST)
/* using a NULL point */
#define AENC_ERR_NULL_PTR ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_NULL_PTR)
/* try to enable or initialize system,device or channel, before configing attribute */
#define AENC_ERR_NOT_CONFIG ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_CONFIG)
/* operation is not supported by NOW */
#define AENC_ERR_NOT_SUPPORT ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_SUPPORT)
/* operation is not permitted ,eg, try to change static attribute */
#define AENC_ERR_NOT_PERM ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_PERM)
/* failure caused by malloc memory */
#define AENC_ERR_NOMEM ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_NOMEM)
/* failure caused by malloc buffer */
#define AENC_ERR_NOBUF ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_NOBUF)
/* no data in buffer */
#define AENC_ERR_BUF_EMPTY ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_EMPTY)
/* no buffer for new data */
#define AENC_ERR_BUF_FULL ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_FULL)
/* system is not ready,had not initialed or loaded*/
#define AENC_ERR_SYS_NOTREADY ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, EN_ERR_SYS_NOTREADY)
/* encoder internal err */
#define AENC_ERR_ENCODER_ERR ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, AENC_ERR_ENCODER_ERR)
/* vqe internal err */
#define AENC_ERR_VQE_ERR ES_DEF_ERR(ES_ID_AENC, EN_ERR_LEVEL_ERROR, AENC_ERR_VQE_ERR)

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif

#endif  // AUDIO_INCLUDE_ES_COMM_AENC_H_
