// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef AUDIO_INCLUDE_ES_COMM_AIO_H_
#define AUDIO_INCLUDE_ES_COMM_AIO_H_

#include "es_common.h"
#include "es_errno.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* End of #ifdef __cplusplus */

#define MAX_AUDIO_FRAME_NUM 300  /*max count of audio frame in Buffer */
#define MAX_AUDIO_POINT_BYTES 4  /*max bytes of one sample point(now 32bit max)*/
#define MAX_VOICE_POINT_NUM 480  /*max sample per frame for voice encode */
#define MAX_AUDIO_POINT_NUM 2048 /*max sample per frame for all encoder(aacplus:2048)*/
#define MAX_AIO_POINT_NUM 4096   /*support 4096 framelen*/
#define MIN_AUDIO_POINT_NUM 80   /*min sample per frame*/
#define MAX_AI_POINT_NUM 2048    /*max sample per frame for all encoder(aacplus:2048)*/

/*max length of audio frame by bytes, one frame contain many sample point */
#define MAX_AUDIO_FRAME_LEN (MAX_AUDIO_POINT_BYTES * MAX_AIO_POINT_NUM)

/*max length of audio stream by bytes */
#define MAX_AUDIO_STREAM_LEN MAX_AUDIO_FRAME_LEN

#define MAX_AI_USRFRM_DEPTH 30 /*max depth of user frame buf */

#define MAX_AUDIO_FILE_PATH_LEN 256
#define MAX_AUDIO_FILE_NAME_LEN 256

/*The VQE EQ Band num.*/
#define VQE_EQ_BAND_NUM 10

#define AI_VQE_MASK_HPF 0x1
#define AI_VQE_MASK_EQ 0x2
#define AI_VQE_MASK_DRC 0x4
#define AI_VQE_MASK_ANR 0x8
#define AI_VQE_MASK_AGC 0x10
#define AI_VQE_MASK_RNR 0x20
#define AI_VQE_MASK_HDR 0x40
#define AI_VQE_MASK_AEC 0x80

#define AO_VQE_MASK_HPF 0x1
#define AO_VQE_MASK_ANR 0x2
#define AO_VQE_MASK_AGC 0x4
#define AO_VQE_MASK_EQ 0x8

typedef enum {
    AUDIO_SAMPLE_RATE_8000 = 8000,   /* 8K samplerate*/
    AUDIO_SAMPLE_RATE_12000 = 12000, /* 12K samplerate*/
    AUDIO_SAMPLE_RATE_11025 = 11025, /* 11.025K samplerate*/
    AUDIO_SAMPLE_RATE_16000 = 16000, /* 16K samplerate*/
    AUDIO_SAMPLE_RATE_22050 = 22050, /* 22.050K samplerate*/
    AUDIO_SAMPLE_RATE_24000 = 24000, /* 24K samplerate*/
    AUDIO_SAMPLE_RATE_32000 = 32000, /* 32K samplerate*/
    AUDIO_SAMPLE_RATE_44100 = 44100, /* 44.1K samplerate*/
    AUDIO_SAMPLE_RATE_48000 = 48000, /* 48K samplerate*/
    AUDIO_SAMPLE_RATE_64000 = 64000, /* 64K samplerate*/
    AUDIO_SAMPLE_RATE_96000 = 96000, /* 96K samplerate*/
    AUDIO_SAMPLE_RATE_BUTT,
} AUDIO_SAMPLE_RATE_E;

typedef enum {
    AUDIO_BIT_WIDTH_8 = 0,  /* 8bit width */
    AUDIO_BIT_WIDTH_16 = 1, /* 16bit width*/
    AUDIO_BIT_WIDTH_24 = 2, /* 24bit width*/
    AUDIO_BIT_WIDTH_32 = 3, /* 32bit width*/
    AUDIO_BIT_WIDTH_BUTT,
} AUDIO_BIT_WIDTH_E;

typedef enum {
    AUDIO_SOUND_MODE_MONO = 0,   /*mono*/
    AUDIO_SOUND_MODE_STEREO = 1, /*stereo*/
    AUDIO_SOUND_MODE_BUTT
} AUDIO_SOUND_MODE_E;

#define AI_EXPAND 0x01
#define AI_CUT 0x02

#ifdef TARGET_ARCH_RISCV
#define ES_AO_HW_BIT_WIDTH AUDIO_BIT_WIDTH_24
#define ES_AO_DEVICE_ID 0
#define ES_HDMI_DEVICE_ID 1
#else
#define ES_AO_HW_BIT_WIDTH AUDIO_BIT_WIDTH_32
#define ES_AO_DEVICE_ID 0
#define ES_HDMI_DEVICE_ID 0
#endif

#define ES_AI_HW_BIT_WIDTH AUDIO_BIT_WIDTH_32
#define ES_AI_HW_SAMPLE_RATE AUDIO_SAMPLE_RATE_48000

#define AUDIO_FD_INVALID (-1)

typedef struct {
    AUDIO_SAMPLE_RATE_E Samplerate; /* sample rate */
    AUDIO_BIT_WIDTH_E Bitwidth;     /* data bitwidth */
    AUDIO_BIT_WIDTH_E HwBitwidth;   /* hw bitwidth */
    AUDIO_SOUND_MODE_E Soundmode;   /* mono or steror */
    ES_U32 FrmNum;                  /* frame num in buf[2,MAX_AUDIO_FRAME_NUM] */
    ES_U32 PtNumPerFrm;             // point num per frame (160/240/320/480/1024/2048/4096)
    ES_U32 ChnCnt;                  /* channle number on FS, valid value:1/2/4/8 */
} AIO_ATTR_S;

typedef struct {
    ES_U32 UsrFrmDepth;
} AI_CHN_PARAM_S;

typedef struct {
    AUDIO_BIT_WIDTH_E Bitwidth;   /*audio frame bitwidth*/
    AUDIO_SOUND_MODE_E Soundmode; /*audio frame momo or stereo mode*/
    ES_U8 *VirAddr;
    ES_U64 TimeStamp; /*audio frame timestamp*/
    ES_U32 Seq;       /*audio frame seq*/
    ES_U32 Len;       /*data lenth per channel in frame*/
} AUDIO_FRAME_S;

typedef struct {
    AUDIO_FRAME_S *RefFrame; /* AEC reference audio frame */
    ES_BOOL Valid;           /* whether frame is valid */
    ES_BOOL SysBind;         /* whether is sysbind */
} AUDIO_AEC_FRAME_S;

typedef struct {
    AUDIO_FRAME_S *Frame; /*frame ptr*/
    ES_U32 Id;            /*frame id*/
} AUDIO_FRAME_INFO_S;

typedef struct {
    ES_U8 ATTRIBUTE *Stream; /* the virtual address of stream */
    ES_U32 Len;              /* stream lenth, by bytes */
    ES_U64 TimeStamp;        /* frame time stamp*/
    ES_U32 Seq;              /* frame seq,if stream is not a valid frame,u32Seq is 0*/
} AUDIO_STREAM_S;

typedef struct {
    ES_U32 ChnTotalNum; /* total number of channel buffer */
    ES_U32 ChnFreeNum;  /* free number of channel buffer */
    ES_U32 ChnBusyNum;  /* busy number of channel buffer */
} AO_CHN_STATE_S;

typedef enum {
    AUDIO_TRACK_NORMAL = 0,
    AUDIO_TRACK_BOTH_LEFT = 1,
    AUDIO_TRACK_BOTH_RIGHT = 2,
    AUDIO_TRACK_EXCHANGE = 3,
    AUDIO_TRACK_MIX = 4,
    AUDIO_TRACK_LEFT_MUTE = 5,
    AUDIO_TRACK_RIGHT_MUTE = 6,
    AUDIO_TRACK_BOTH_MUTE = 7,

    AUDIO_TRACK_BUTT
} AUDIO_TRACK_MODE_E;

typedef struct {
    ES_BOOL Fade;
    ES_U32 FadeInRate;
    ES_U32 FadeOutRate;
} AUDIO_FADE_S;

// TODO(yiguo): Modify it adapt es win2030.
/**Defines the configure parameters of AEC.*/
typedef struct esAI_AEC_CONFIG_S {
    ES_BOOL bUsrMode;           /* mode 0: auto mode 1: mannual.*/
    ES_S8 s8CngMode;            /* cozy noisy mode: 0 close, 1 open, recommend 1*/
    ES_S8 s8NearAllPassEnergy;  // the far-end energy threshold for judging whether unvarnished transmission: 0 -59dBm0,
                                // 1 -49dBm0, 2 -39dBm0, recommend 1
    ES_S8 s8NearCleanSupEnergy;  // the energy threshold for compelling reset of near-end signal: 0 12dB, 1 15dB, 2
                                 // 18dB, recommend 2 */

    ES_S16 s16DTHnlSortQTh; /* the threshold of judging single or double talk, recommend 16384, [0, 32767] */

    ES_S16 s16EchoBandLow;    // voice processing band1, low frequency parameter, [1, 63) for 8k, [1, 127) for 16k,
                              // recommend 10 */
    ES_S16 s16EchoBandHigh;   // voice processing band1, high frequency parameter, (s16EchoBandLow, 63] for 8k,
                              // (s16EchoBandLow, 127] for 16k, recommend 41 */
                              // s16EchoBandHigh must be greater than s16EchoBandLow */
    ES_S16 s16EchoBandLow2;   // voice processing band2, low frequency parameter, [1, 63) for 8k, [1, 127) for 16k,
                              // recommend 47 */
    ES_S16 s16EchoBandHigh2;  // voice processing band2, high frequency parameter, (s16EchoBandLow2, 63] for 8k,
                              // (s16EchoBandLow2, 127] for 16k, recommend 72 */
                              // s16EchoBandHigh2 must be greater than s16EchoBandLow2 */

    ES_S16 s16ERLBand[6];  // ERL protect area, [1, 63] for 8k, [1, 127] for 16k, frequency band calculated by
                           // s16ERLBand * 62.5
                           // besides, s16ERLBand[n+1] should be greater than s16ERLBand[n]
    ES_S16 s16ERL[7];  // ERL protect value of ERL protect area, the smaller its value, the more strength its protect
                       // ability,[0, 18]*/
    ES_S16 s16VioceProtectFreqL;  // protect area of near-end low frequency, [1, 63) for 8k, [1, 127) for 16k, recommend
                                  // 3
    ES_S16 s16VioceProtectFreqL1;  // protect area of near-end low frequency1, (s16VioceProtectFreqL, 63] for 8k,
                                   // (s16VioceProtectFreqL, 127] for 16k, recommend 6
    ES_S32 s32Reserved;            // s16VioceProtectFreqL1 must be greater than s16VioceProtectFreqL
} AI_AEC_CONFIG_S;

/**Defines the configure parameters of ANR.*/
typedef struct {
    ES_BOOL UsrMode; /* mode 0: auto, mode 1: manual.*/

    ES_U8 Mode; /* noise reduce mode, range: [0, 3] */
} AUDIO_ANR_CONFIG_S;

/**Defines the configure parameters of AGC.*/
typedef struct {
    ES_BOOL UsrMode; /* mode 0: auto, mode 1: manual.*/

    ES_S8 TargetLevel;     /* target level, range: [-30, 0]dB */
    ES_U8 CompressionGain; /* compression gain, range: [0, 90]dB */
    ES_U8 LimiterEnable;   /* limiter enable or disable, range: [0:disable, 1:enable] */
} AUDIO_AGC_CONFIG_S;

/**Defines the configure parameters of AEC.*/
typedef struct {
    ES_BOOL UsrMode; /* mode 0: auto, mode 1: manual.*/

    ES_U8 EchoMode; /* echo mode, range: [0, 1] */
} AUDIO_AEC_CONFIG_S;

typedef struct {
    ES_BOOL UsrMode; /* mode 0: auto mode 1: mannual.*/

    ES_U16 HpfFreq; /* freq to be processed, range: [80, 200]Hz */
} AUDIO_HPF_CONFIG_S;

// TODO(yiguo): Modify it adapt es win2030.
typedef struct esAI_RNR_CONFIG_S {
    ES_BOOL UsrMode; /* mode 0: auto, mode 1: mannual. */

    ES_S32 NrMode; /* mode 0: floor noise; 1:ambient noise */

    ES_S32 MaxNrLevel; /* max NR level range:[2,20]dB */

    ES_S32 NoiseThresh; /* noise threshold, range:[-80, -20] */
} AI_RNR_CONFIG_S;

typedef struct {
    ES_S8 GaindB[VQE_EQ_BAND_NUM];  // EQ band, include 100,200,250,350,500,800,1.2k,2.5k,4k,8k in turn,
                                    // range: [-15, 15]dB
} AUDIO_EQ_CONFIG_S;

/**Defines the configure parameters of UPVQE work state.*/
typedef enum {
    VQE_WORKSTATE_COMMON = 0, /* common environment, Applicable to the family of voice calls. */
    VQE_WORKSTATE_MUSIC = 1,  /* music environment , Applicable to the family of music environment. */
    VQE_WORKSTATE_NOISY = 2,  /* noisy environment , Applicable to the noisy voice calls.  */
} VQE_WORKSTATE_E;

// TODO(yiguo): Modify it adapt es win2030.
/* HDR Set CODEC GAIN Function Handle type */
typedef ES_S32 (*pFuncGainCallBack)(ES_S32 s32SetGain);

typedef struct esAI_HDR_CONFIG_S {
    ES_BOOL UsrMode; /* mode 0: auto mode 1: mannual.*/

    ES_S32 MinGaindB; /* the minimum of MIC(AI) CODEC gain, [0, 120]*/
    ES_S32 MaxGaindB; /* the maximum of MIC(AI) CODEC gain, [0, 120]*/

    ES_S32 MicGaindB;           /* the current gain of MIC(AI) CODEC,[s32MinGaindB, s32MaxGaindB]*/
    ES_S32 MicGainStepdB;       /* the step size of gain adjustment, [1, 3], recommemd 2 */
    pFuncGainCallBack Callback; /* the callback function pointer of CODEC gain adjustment */
} AI_HDR_CONFIG_S;

typedef struct {
    ES_BOOL UsrMode; /* enable user mode or not,default 0: disable user mode,1: user mode.*/

    ES_S8 Threshold; /* The value(dB) above which the compression starts: [-90, 0]dB */
    ES_U8 Knee;      /* The value(dB) above which the knee region starts: [0, 90]dB */
    ES_U8 Ratio;     /* The input/output dB ratio after the knee region: [1, 10] */
    ES_U16 Attack;   /* The time to increase the gain by 10dB: [10, 500]ms */
    ES_U16 Release;  /* The time to reduce the gain by 10dB: [10, 500]ms */
    ES_S8 PostGain;  /* The static boost value in output: [-10, 10]dB */
} AI_DRC_CONFIG_S;

/**Defines the configure parameters of AI VQE.*/
typedef struct {
    ES_U32 OpenMask;

    ES_S32 WorkSampleRate; /* Sample Rate:8kHz/16kHz/32kHz/48kHz */
    ES_S32 FrameSamples;   /* VQE frame length:80-4096 */
    VQE_WORKSTATE_E WorkState;

    // ES_S32 InChNum;
    // ES_S32 OutChNum;

    AUDIO_HPF_CONFIG_S HpfConfig;
    AUDIO_EQ_CONFIG_S EqConfig;
    AI_DRC_CONFIG_S DrcConfig;
    AUDIO_ANR_CONFIG_S AnrConfig;
    AUDIO_AGC_CONFIG_S AgcConfig;
    AUDIO_AEC_CONFIG_S AecConfig;
} AI_VQE_CONFIG_S;

typedef struct {
    ES_U32 OpenMask;

    ES_S32 WorkSampleRate; /* Sample Rate: 8kHz/16kHz/32kHz/48kHz. */
    ES_S32 FrameSamples;   /* VQE frame length: 80-4096 */
    VQE_WORKSTATE_E WorkState;

    AUDIO_HPF_CONFIG_S HpfConfig;
    AUDIO_ANR_CONFIG_S AnrConfig;
    AUDIO_AGC_CONFIG_S AgcConfig;
    AUDIO_EQ_CONFIG_S EqConfig;
} AO_VQE_CONFIG_S;

/*Defines the configure parameters of AI saving file.*/
typedef struct {
    ES_BOOL Cfg;
    ES_CHAR FilePath[MAX_AUDIO_FILE_PATH_LEN];
    ES_CHAR FileName[MAX_AUDIO_FILE_NAME_LEN];
    ES_U32 FileSize; /*in KB*/
} AUDIO_SAVE_FILE_INFO_S;

/*Defines whether the file is saving or not .*/
typedef struct {
    ES_BOOL Saving;
} AUDIO_FILE_STATUS_S;

typedef enum esEN_AIO_ERR_CODE_E {
    AIO_ERR_VQE_ERR = 65, /*vqe error*/
} AIO_ERR_CODE_E;

/* invlalid device ID */
#define AI_ERR_INVALID_DEVID ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_DEVID)
/* invlalid channel ID */
#define AI_ERR_INVALID_CHNID ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_CHNID)
/* at lease one parameter is illagal ,eg, an illegal enumeration value  */
#define AI_ERR_ILLEGAL_PARAM ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_ILLEGAL_PARAM)
/* using a NULL point */
#define AI_ERR_NULL_PTR ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_NULL_PTR)
/* try to enable or initialize system,device or channel, before configing attribute */
#define AI_ERR_NOT_CONFIG ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_CONFIG)
/* operation is not supported by NOW */
#define AI_ERR_NOT_SUPPORT ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_SUPPORT)
/* operation is not permitted ,eg, try to change stati attribute */
#define AI_ERR_NOT_PERM ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_PERM)
/* the devide is not enabled  */
#define AI_ERR_NOT_ENABLED ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_UNEXIST)
/* failure caused by malloc memory */
#define AI_ERR_NOMEM ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_NOMEM)
/* failure caused by malloc buffer */
#define AI_ERR_NOBUF ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_NOBUF)
/* no data in buffer */
#define AI_ERR_BUF_EMPTY ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_EMPTY)
/* no buffer for new data */
#define AI_ERR_BUF_FULL ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_FULL)
/* system is not ready,had not initialed or loaded*/
#define AI_ERR_SYS_NOTREADY ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_SYS_NOTREADY)

#define AI_ERR_BUSY ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, EN_ERR_BUSY)
/* vqe  err */
#define AI_ERR_VQE_ERR ES_DEF_ERR(ES_ID_AI, EN_ERR_LEVEL_ERROR, AIO_ERR_VQE_ERR)

/* invlalid device ID */
#define AO_ERR_INVALID_DEVID ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_DEVID)
/* invlalid channel ID */
#define AO_ERR_INVALID_CHNID ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_CHNID)
/* at lease one parameter is illagal ,eg, an illegal enumeration value  */
#define AO_ERR_ILLEGAL_PARAM ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_ILLEGAL_PARAM)
/* using a NULL point */
#define AO_ERR_NULL_PTR ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_NULL_PTR)
/* try to enable or initialize system,device or channel, before configing attribute */
#define AO_ERR_NOT_CONFIG ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_CONFIG)
/* operation is not supported by NOW */
#define AO_ERR_NOT_SUPPORT ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_SUPPORT)
/* operation is not permitted ,eg, try to change stati attribute */
#define AO_ERR_NOT_PERM ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_PERM)
/* the devide is not enabled  */
#define AO_ERR_NOT_ENABLED ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_UNEXIST)
/* failure caused by malloc memory */
#define AO_ERR_NOMEM ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_NOMEM)
/* failure caused by malloc buffer */
#define AO_ERR_NOBUF ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_NOBUF)
/* no data in buffer */
#define AO_ERR_BUF_EMPTY ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_EMPTY)
/* no buffer for new data */
#define AO_ERR_BUF_FULL ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_FULL)
/* system is not ready,had not initialed or loaded*/
#define AO_ERR_SYS_NOTREADY ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_SYS_NOTREADY)

#define AO_ERR_BUSY ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, EN_ERR_BUSY)
/* vqe  err */
#define AO_ERR_VQE_ERR ES_DEF_ERR(ES_ID_AO, EN_ERR_LEVEL_ERROR, AIO_ERR_VQE_ERR)

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* End of #ifdef __cplusplus */

#endif  // AUDIO_INCLUDE_ES_COMM_AIO_H_
