// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef AUDIO_INCLUDE_ES_DEBUG_H_
#define AUDIO_INCLUDE_ES_DEBUG_H_

#ifndef __KERNEL__
#include <stdio.h>
#include <stdarg.h>
#endif

#include "AudioLog.h"
#include "es_type.h"
#include "es_common.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* __cplusplus */

#define _EX__FILE_LINE(fxx, lxx) "[File]:" fxx "\n[Line]:" #lxx "\n[Info]:"
#define EX__FILE_LINE(fxx, lxx) _EX__FILE_LINE(fxx, lxx)
#define __FILE_LINE__ EX__FILE_LINE(__FILE__, __LINE__)

#define ES_DBG_EMERG 0  /* system is unusable                   */
#define ES_DBG_ALERT 1  /* action must be taken immediately     */
#define ES_DBG_CRIT 2   /* critical conditions                  */
#define ES_DBG_ERR 3    /* error conditions                     */
#define ES_DBG_WARN 4   /* warning conditions                   */
#define ES_DBG_NOTICE 5 /* normal but significant condition     */
#define ES_DBG_INFO 6   /* informational                        */
#define ES_DBG_DEBUG 7  /* debug-level messages                 */

typedef struct esLOG_LEVEL_CONF_S {
    MOD_ID_E enModId;
    ES_S32 s32Level;
    ES_CHAR cModName[16];
} LOG_LEVEL_CONF_S;

#ifndef __KERNEL__
/******************************************************************************
** For User Mode : ES_PRINT, ES_ASSERT, ES_TRACE
******************************************************************************/

#define ES_PRINT printf

/* #ifdef ES_DEBUG */
#if 1
/* Using samples:   ES_ASSERT(x>y); */
#define ES_ASSERT(expr)                            \
    do {                                           \
        if (!(expr)) {                             \
            printf("\nASSERT at:\n"                \
                   "  >Function : %s\n"            \
                   "  >Line No. : %d\n"            \
                   "  >Condition: %s\n",           \
                   __FUNCTION__, __LINE__, #expr); \
            _exit(-1);                             \
        }                                          \
    } while (0)

static inline const char* modId_to_string(MOD_ID_E enModId)
{
    switch (enModId) {
    case ES_ID_ADEC:
        return "ADEC";
        break;
    case ES_ID_AENC:
        return "AENC";
        break;
    case ES_ID_AI:
        return "AI";
        break;
    case ES_ID_AO:
        return "AO";
        break;
    default:
        return "Unknow";
        break;
    }
}
/* Using samples:
** ES_TRACE(ES_DBG_DEBUG, ES_ID_CMPI, "Test %d, %s\n", 12, "Test");
**/
#ifdef TARGET_ARCH_RISCV
#include "es_log.h"
#define ES_TRACE(level, enModId, fmt, args...)                                                                      \
    do {                                                                                                            \
        switch (level) {                                                                                            \
        case ES_DBG_DEBUG:                                                                                          \
            es_log_print(enModId, ES_LOG_DBG, "[%s::%s(): %d]" fmt "\n", modId_to_string(enModId), __FUNCTION__,    \
                         __LINE__, ##args);                                                                         \
            break;                                                                                                  \
        case ES_DBG_INFO:                                                                                           \
            es_log_print(enModId, ES_LOG_INFO, "[%s::%s(): %d]" fmt "\n", modId_to_string(enModId), __FUNCTION__,   \
                         __LINE__, ##args);                                                                         \
            break;                                                                                                  \
        case ES_DBG_NOTICE:                                                                                         \
            es_log_print(enModId, ES_LOG_NOTICE, "[%s::%s(): %d]" fmt "\n", modId_to_string(enModId), __FUNCTION__, \
                         __LINE__, ##args);                                                                         \
            break;                                                                                                  \
        case ES_DBG_WARN:                                                                                           \
            es_log_print(enModId, ES_LOG_WARN, "[%s::%s(): %d]" fmt "\n", modId_to_string(enModId), __FUNCTION__,   \
                         __LINE__, ##args);                                                                         \
            break;                                                                                                  \
        case ES_DBG_ERR:                                                                                            \
            es_log_print(enModId, ES_LOG_ERR, "[%s::%s(): %d]" fmt "\n", modId_to_string(enModId), __FUNCTION__,    \
                         __LINE__, ##args);                                                                         \
            break;                                                                                                  \
        case ES_DBG_CRIT:                                                                                           \
            es_log_print(enModId, ES_LOG_FATAL, "[%s::%s(): %d]" fmt "\n", modId_to_string(enModId), __FUNCTION__,  \
                         __LINE__, ##args);                                                                         \
            break;                                                                                                  \
        default:                                                                                                    \
            break;                                                                                                  \
        }                                                                                                           \
    } while (0)
#else
#define ES_TRACE(level, enModId, fmt, args...)                                                                   \
    do {                                                                                                         \
        switch (level) {                                                                                         \
        case ES_DBG_DEBUG:                                                                                       \
            log_log(LOG_DEBUG, LOG_TAG, "[%s::%s(): %d] " fmt, modId_to_string(enModId), __FUNCTION__, __LINE__, \
                    ##args);                                                                                     \
            break;                                                                                               \
        case ES_DBG_INFO:                                                                                        \
            log_log(LOG_INFO, LOG_TAG, "[%s::%s(): %d] " fmt, modId_to_string(enModId), __FUNCTION__, __LINE__,  \
                    ##args);                                                                                     \
            break;                                                                                               \
        case ES_DBG_WARN:                                                                                        \
            log_log(LOG_WARN, LOG_TAG, "[%s::%s(): %d] " fmt, modId_to_string(enModId), __FUNCTION__, __LINE__,  \
                    ##args);                                                                                     \
            break;                                                                                               \
        case ES_DBG_ERR:                                                                                         \
            log_log(LOG_ERROR, LOG_TAG, "[%s::%s(): %d] " fmt, modId_to_string(enModId), __FUNCTION__, __LINE__, \
                    ##args);                                                                                     \
            break;                                                                                               \
        default:                                                                                                 \
            break;                                                                                               \
        }                                                                                                        \
    } while (0)
#endif

#else
#define ES_ASSERT(expr)
#define ES_TRACE(level, enModId, fmt...)
#endif

#else
/******************************************************************************
** For Linux Kernel : ES_PRINT, ES_ASSERT, ES_TRACE
******************************************************************************/
// #include "es_osal.h"
#define ES_PRINT osal_printk

extern ES_S32 ES_ChkLogLevel(ES_S32 s32Levle, MOD_ID_E enModId);

int ES_LOG(ES_S32 level, MOD_ID_E enModId, const char* fmt, ...) __attribute__((format(printf, 3, 4)));

/* #ifdef ES_DEBUG */
#if 1
/* Using samples:   ES_ASSERT(x>y); */
#define ES_ASSERT(expr)                                \
    do {                                               \
        if (!(expr)) {                                 \
            osal_panic("\nASSERT at:\n"                \
                       "  >Function : %s\n"            \
                       "  >Line No. : %d\n"            \
                       "  >Condition: %s\n",           \
                       __FUNCTION__, __LINE__, #expr); \
        }                                              \
    } while (0)

/* Using samples:
** ES_TRACE(ES_DBG_DEBUG, ES_ID_CMPI, "Test %d, %s\n", 12, "Test");
**/
#define ES_TRACE ES_LOG
#else
#define ES_ASSERT(expr)
#define ES_TRACE(level, enModId, fmt...)
#endif

#endif /* end of __KERNEL__ */

#define ES_PRINT_BLOCK(pu8Datablock, u32Length)                                                                     \
    {                                                                                                               \
        ES_U32 u32ii = 0;                                                                                           \
        ES_U8* pu8VirAddr = (ES_U8*)(pu8Datablock);                                                                 \
        if (ES_NULL != pu8VirAddr) {                                                                                \
            ES_PRINT("\n[Func]:%s [Line]:%d [Info]:%s\n", __FUNCTION__, __LINE__, #pu8Datablock);                   \
            for (u32ii = 0; u32ii < (u32Length);) {                                                                 \
                ES_PRINT(" %02X", *pu8VirAddr);                                                                     \
                pu8VirAddr++;                                                                                       \
                u32ii++;                                                                                            \
                if (0 == (u32ii % 16)) ES_PRINT("\n");                                                              \
            }                                                                                                       \
            ES_PRINT("\n\n");                                                                                       \
        } else {                                                                                                    \
            ES_PRINT("\n[Func]:%s [Line]:%d [Info]:pointer(%s) is null!\n", __FUNCTION__, __LINE__, #pu8Datablock); \
        }                                                                                                           \
    }

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif  // AUDIO_INCLUDE_ES_DEBUG_H_
